-- leds.adb

with Ada.Unchecked_Conversion;

with LCD;
with MUX_Timer;

package body LEDs is


   function As_Word is new Ada.Unchecked_Conversion
     (Source => User_LED, Target => Word);


   procedure On (This : User_LED) is
   begin
      GPIOD.BSRR := As_Word (This);
   end On;


   procedure Off (This : User_LED) is
   begin
      GPIOD.BSRR := Shift_Left (As_Word (This), 16);
   end Off;


   All_LEDs_On  : constant Word := Green'Enum_Rep or Red'Enum_Rep or
                                   Blue'Enum_Rep  or Orange'Enum_Rep;

   pragma Compile_Time_Error
     (All_LEDs_On /= 16#F000#,
      "Invalid representation for All_LEDs_On");

   All_LEDs_Off : constant Word := Shift_Left (All_LEDs_On, 16);


   procedure All_Off is
   begin
      GPIOD.BSRR := All_LEDs_Off;
   end All_Off;


   procedure All_On is
   begin
      GPIOD.BSRR := All_LEDs_On;
   end All_On;

   procedure Init_GPIO(GPIO : in out GPIO_Register; I : Natural;
		       Mode : Bits_2;
		       Otype : Bits_1;
		       Speed : Bits_2;
		      PUtype : Bits_2) is
   begin
      GPIO.MODER   (I) := Mode;
      GPIO.OTYPER  (I) := Otype;
      GPIO.OSPEEDR (I) := Speed;
      GPIO.PUPDR   (I) := PUtype;
   end Init_GPIO;

   procedure Init_GPIO(GPIO : in out GPIO_Register; LO : Natural; HI : Natural;
		       Mode : Bits_2;
		       Otype : Bits_1;
		       Speed : Bits_2;
		       PUtype : Bits_2) is
   begin
      for I in LO..HI loop
	 Init_GPIO(GPIO, I, Mode, Otype, Speed, PUtype);
      end loop;
   end Init_GPIO;

   procedure Init_GPIO(GPIO : in out GPIO_Register; Sel : Bits_16;
		       Mode : Bits_2;
		       Otype : Bits_1;
		       Speed : Bits_2;
		       PUtype : Bits_2) is
   begin
      for I in Sel'Range loop
	 if Sel(I) then
	    Init_GPIO(GPIO, I, Mode, Otype, Speed, PUtype);
	 end if;
      end loop;
   end Init_GPIO;

   procedure Initialize is
      RCC_AHB1ENR_GPIOB : constant Word := 2#0_0010#;
      RCC_AHB1ENR_GPIOC : constant Word := 2#0_0100#;
      RCC_AHB1ENR_GPIOD : constant Word := 2#0_1000#;
      RCC_AHB1ENR_GPIOE : constant Word := 2#1_0000#;
      GPIO_B_Sel        : constant HWord := 2#1111_0000_0010_0011#; --12-15, 5, 0-1
      GPIO_C_Sel        : constant HWord := 2#0000_1001_0101_0110#;
      GPIO_D_Sel        : constant HWord := 2#1111_0000_1100_1100#; --12-15, 6-7, 2-3
   begin
      --  Enable clock for GPIO-D and GPIO-E
      RCC.AHB1ENR := RCC.AHB1ENR
	or RCC_AHB1ENR_GPIOB
	or RCC_AHB1ENR_GPIOC
        or RCC_AHB1ENR_GPIOD
        or RCC_AHB1ENR_GPIOE;
      -- Configure B segment outputs and digit outputs
      Init_GPIO(GPIOB, B16_From_HW(GPIO_B_Sel),
		Mode_OUT,
		Type_PP,
		Speed_2MHz,
		No_Pull);
      -- Configure C segment outputs and LED outputs
      Init_GPIO(GPIOC, B16_From_HW(GPIO_C_Sel),
		Mode_OUT,
		Type_PP,
		Speed_2MHz,
		No_Pull);
      -- Configure D segment outputs
      Init_GPIO(GPIOD, B16_From_HW(GPIO_D_Sel),
		Mode_OUT,
		Type_PP,
		Speed_2MHz,
		No_Pull);
      --  Configure PE4-9 (4-7 LCD data, 8 LCD RW, 9 LCD E
      Init_GPIO(GPIOE, 4, 9,
		Mode_OUT,
		Type_PP,
		Speed_25MHz,
		No_Pull);
      --  Configure PE2
      Init_GPIO(GPIOE, 2,
		Mode_OUT,
		Type_PP,
		Speed_25MHz,
		No_Pull);
      --  Configure PE15
      Init_GPIO(GPIOE, 15,
		Mode_OUT,
		Type_PP,
		Speed_2MHz,
		No_Pull);
      -- Configure PE10-13 (button inputs)
      Init_GPIO(GPIOE, MUX_Timer.B_CLEAR, MUX_Timer.B_FAST,
		Mode_IN,
		Type_PP,      -- don't care
		Speed_2MHz,   -- don't care
		Pull_Up);
   end Initialize;


begin
   Initialize;
   MUX_Timer.Init;
   LCD.LCD_Init;
end LEDs;
